package fr.unistra.sil.erp.back.repository.registry;

import fr.unistra.sil.erp.back.model.RegistryEntry;
import fr.unistra.sil.erp.back.repository.IRegistryRepository;
import fr.unistra.sil.erp.back.repository.SqliteRepository;
import org.springframework.stereotype.Repository;

import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author antoine.beauvais
 */
@Repository
public class SqliteRegistryRepository extends SqliteRepository implements IRegistryRepository {
    /**
     * Query used to retrieve the registry.
     */
    private static final String SQL_GETREGISTRY =
            "SELECT r.id AS id, r.dt AS date, r.type AS typeId, " +
                    "t.name AS type, " +
                    "r.account_id AS accountId, a.name AS account, " +
                    "r.debit AS debit, r.credit AS credit, " +
                    "r.remarks AS remarks FROM Registry r " +
                    "LEFT JOIN accounts a ON r.account_id = a.id " +
                    "LEFT JOIN transaction_types t ON r.type = t.id";

    /**
     * Class constructor.
     * @param conn
     */
    public SqliteRegistryRepository(Connection conn) {
        super(conn);
    }

    @Override
    public List<RegistryEntry> getRegistry() {
        ResultSet rs = this.query(SQL_GETREGISTRY);
        if (rs == null)
            return null;

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<RegistryEntry> res = new ArrayList<>();
        try {
            while (rs.next()) {
                RegistryEntry r = new RegistryEntry(rs.getInt("id"),
                        rs.getInt("typeID"), rs.getString("type"),
                        formatter.parse(rs.getString("date")),
                        rs.getInt("accountId"), rs.getString("account"),
                        rs.getBigDecimal("debit"), rs.getBigDecimal("credit"),
                        rs.getString("remarks"));
                res.add(r);
            }
        } catch (SQLException | ParseException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return res;
    }

    /**
     *
     * @param entry
     * @return
     */
    @Override
    public RegistryEntry store(RegistryEntry entry) {
        String query = "INSERT INTO registry (dt, type, account_id, debit, credit, remarks) VALUES (?, ?, ?, ?, ?, ?);";
        PreparedStatement ps;
        LocalDateTime ldt = entry.getDatetime().toInstant()
                .atZone(ZoneId.of("CET"))
                .toLocalDateTime();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        try {
            ps = this.conn.prepareStatement(query);
            ps.setString(1, ldt.format(formatter));
            ps.setInt(2, entry.getTransactionType());
            ps.setInt(3, entry.getAccountId());
            ps.setBigDecimal(4, entry.getDebit());
            ps.setBigDecimal(5, entry.getCredit());
            ps.setString(6, entry.getRemarks());
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }
        try {
            ps.executeUpdate();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to create registry: " + ex.getMessage());
            return null;
        }
        int id;
        query = "SELECT MAX(id) AS max_id FROM registry";
        ResultSet rs;
        try {
            ps = this.conn.prepareStatement(query);
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to find registry id: " + ex.getMessage());
            return null;
        }
        try {
            id = rs.getInt("max_id");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to parse registry id: " + ex.getMessage());
            return null;
        }
        return new RegistryEntry(
                id,
                entry.getTransactionType(),
                entry.getDatetime(),
                entry.getAccountId(),
                entry.getDebit(),
                entry.getCredit(),
                entry.getRemarks()
        );
    }
}
